
----------------------------------------------------------------------------------
--                          TEST FPGA BLOCK
--                +------------------------------+       ||   +-------+
--                |+---------+ 9bit+------++---+ | 16bit ||<=>| MEMC  |
--   SpW DSin <====| SpW I/F |<=/==| RMAP ||BUS|<====/==>||   +-------+
--   SpW DSout ===>|         | 9bit|      ||I/F| |       ||   +-------+
--                ||         |==/=>|      ||MST| |       ||<=>| EBUSC |
--                |+---------+     +------++---+ |       ||   +-------+
--        RMAP IP +------------------------------+     IP BUS
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;


entity spw_fpga_dio is
	port (
		clk : in std_logic;
		reset_n : in std_logic;
		din : in std_logic;
		sin : in std_logic;
		dout : out std_logic;
		sout : out std_logic;
		--ebus
		ebus_adr : inout std_logic_vector (24 downto 0);
		ebus_d : inout std_logic_vector (15 downto 0);
		ebus_ena : inout std_logic;
		ebus_wrn : inout std_logic;
		ebus_rdn : inout std_logic;
		ebus_req : in std_logic;
		ebus_grant : out std_logic;
		ebus_done : inout std_logic;
		-- SDRAM Interface
		cko : out std_logic;		
		cke : out std_logic;
		ceb : out std_logic;
		rasb : out std_logic;
		casb : out std_logic;
		web : out std_logic;
		dm : out std_logic;
		addr : out std_logic_vector (12 downto 0);
		ba : out std_logic_vector (1 downto 0);
		dq : inout std_logic_vector (7 downto 0);
		led_out : out std_logic_vector (3 downto 0);
		test : out std_logic_vector (3 downto 0)
	);
end spw_fpga_dio;

architecture behavioral of spw_fpga_dio is

	component spw_if is
		port (
			clk : in std_logic;
			tx_clk : in std_logic;
			rx_clk : in std_logic;
			reset : in std_logic;
			
			tx_fifo_we : in std_logic;
			tx_fifo_data_in : in std_logic_vector (8 downto 0);
			tx_fifo_full : out std_logic;
			tx_fifo_data_num : out std_logic_vector (5 downto 0);
			rx_fifo_oe : in std_logic;
			rx_fifo_data_out : out std_logic_vector (8 downto 0);
			rx_fifo_full : out std_logic;
			rx_fifo_data_num : out std_logic_vector (5 downto 0);
			
			tick_in : in std_logic;
			time_in : in std_logic_vector (5 downto 0);
			control_flags_in : in std_logic_vector (1 downto 0);
			tick_out: out std_logic;
			time_out: out std_logic_vector (5 downto 0);
			control_flags_out: out std_logic_vector (1 downto 0);
			
			link_start: in std_logic;
			link_disable: in std_logic;
			auto_start: in std_logic;
			link_status : out std_logic_vector (7 downto 0);
			error_status : out std_logic_vector (7 downto 0);
			txclk_sel : in std_logic_vector (5 downto 0);
			
			dout: out std_logic;
			sout: out std_logic;
			din: in std_logic;
			sin: in std_logic
		);
	end component;
	
	--clock and reset
	signal reset : std_logic;
	
	signal clk25m_wire : std_logic;
	signal clk50m_wire : std_logic;
	signal clk100m_wire : std_logic;
	signal clk100mb_wire : std_logic;
	signal clk120m_wire : std_logic;
	
	signal fpga_clk : std_logic;
	signal clk25meg : std_logic;
	signal clk50meg : std_logic;
	signal clk100meg : std_logic;
	signal clk100megb : std_logic;
	signal clk120meg : std_logic;
	

	
	--spw_if
	signal tick_in: std_logic;
	signal time_in: std_logic_vector (5 downto 0);
	signal control_flags_in: std_logic_vector (1 downto 0);
	signal tick_out: std_logic;
	signal time_out: std_logic_vector (5 downto 0);
	signal control_flags_out: std_logic_vector (1 downto 0);
		
	signal tx_fifo_we : std_logic;
	signal tx_fifo_data_in : std_logic_vector (8 downto 0);
	signal tx_fifo_full : std_logic;
	signal tx_fifo_data_num : std_logic_vector (5 downto 0);
	signal rx_fifo_oe : std_logic;
	signal rx_fifo_data_out : std_logic_vector (8 downto 0);
	signal rx_fifo_full : std_logic;
	signal rx_fifo_data_num : std_logic_vector (5 downto 0);
		
	signal link_start: std_logic;
	signal link_disable: std_logic;
	signal auto_start: std_logic;
	signal link_status : std_logic_vector (7 downto 0);
	signal error_status : std_logic_vector (7 downto 0);
	signal txclk_sel : std_logic_vector (5 downto 0);
		
	
	
	signal spw_link_status : std_logic_vector (7 downto 0);
	signal spw_error_status : std_logic_vector (7 downto 0);
	
	signal clk_ibufg_o : std_logic;
	signal init_wait_count : integer := 0;
	
	signal counter : integer :=0;
begin

	reset <= not reset_n;

	spw_clock : DCM
		generic map (
			CLKDV_DIVIDE => 2.0,		--  Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
										--     7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
			CLKFX_DIVIDE => 3,			--  Can be any interger from 1 to 32
			CLKFX_MULTIPLY => 10,		--  Can be any integer from 1 to 32
			CLKIN_DIVIDE_BY_2 => FALSE,	--  TRUE/FALSE to enable CLKIN divide by two feature
			CLKIN_PERIOD => 20.0,		--  Specify period of input clock
			CLKOUT_PHASE_SHIFT => "NONE",	--  Specify phase shift of NONE, FIXED or VARIABLE
			CLK_FEEDBACK => "1X",			--  Specify clock feedback of NONE, 1X or 2X
			DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS", --  SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
											--     an integer from 0 to 15
			DFS_FREQUENCY_MODE => "LOW",	--  HIGH or LOW frequency mode for frequency synthesis
			DLL_FREQUENCY_MODE => "LOW",	--  HIGH or LOW frequency mode for DLL
			DUTY_CYCLE_CORRECTION => TRUE,	--  Duty cycle correction, TRUE or FALSE
			FACTORY_JF => X"C080",			--  FACTORY JF Values
			PHASE_SHIFT => 0,				--  Amount of fixed phase shift from -255 to 255
			STARTUP_WAIT => FALSE)			--  Delay configuration DONE until DCM LOCK, TRUE/FALSE
		port map (
			CLK0 => clk50m_wire,		-- 0 degree DCM CLK ouptput
		--	CLK180 => CLK180,			-- 180 degree DCM CLK output
		--	CLK270 => CLK270,			-- 270 degree DCM CLK output
			CLK2X => clk100m_wire,		-- 2X DCM CLK output
			CLK2X180 => clk100mb_wire,	-- 2X, 180 degree DCM CLK out
		--	CLK90 => CLK90,				-- 90 degree DCM CLK output
			CLKDV => clk25m_wire,		-- Divided DCM CLK out (CLKDV_DIVIDE)
			CLKFX => clk120m_wire,		-- DCM CLK synthesis out (M/D)
		--	CLKFX180 => clk100mb_wire,	-- 180 degree CLK synthesis out
		--	LOCKED => LOCKED,			-- DCM LOCK status output
		--	PSDONE => PSDONE,			-- Dynamic phase adjust done output
		--	STATUS => STATUS,			-- 8-bit DCM status bits output
			CLKFB => fpga_clk,			-- DCM clock feedback
			CLKIN => clk_ibufg_o				-- Clock input (from IBUFG, BUFG or DCM)
		--	PSCLK => PSCLK,				-- Dynamic phase adjust clock input
		--	PSEN => PSEN,				-- Dynamic phase adjust enable input
		--	PSINCDEC => PSINCDEC,		-- Dynamic phase adjust increment/decrement
		--	RST => reset				-- DCM asynchronous reset input
		);
	
	clk_ibufg : IBUFG port map(I=>clk, O=>clk_ibufg_o);
	clk25m_buf : BUFG port map (I => clk25m_wire, O => clk25meg);	-- not used
	clk50m_buf : BUFG port map (I => clk50m_wire, O => fpga_clk);
	clk100m_buf : BUFG port map (I => clk100m_wire, O => clk100meg);
	clk100mb_buf : BUFG port map (I => clk100mb_wire, O => clk100megb);
	clk120m_buf : BUFG port map (I => clk120m_wire, O => clk120meg);
	
	clk50meg <= fpga_clk;

	spw : spw_if
	port map (
		clk => clk100meg,
		tx_clk => clk100meg,
		rx_clk => clk120meg,
		reset => reset,
		
		tx_fifo_we => tx_fifo_we,
		tx_fifo_data_in => tx_fifo_data_in,
		tx_fifo_full => tx_fifo_full,
		tx_fifo_data_num => tx_fifo_data_num,
		rx_fifo_oe => rx_fifo_oe,
		rx_fifo_data_out => rx_fifo_data_out,
		rx_fifo_data_num => rx_fifo_data_num,
		
		tick_in => tick_in,
		time_in => time_in,
		control_flags_in => control_flags_in,
		tick_out => tick_out,
		time_out => time_out,
		control_flags_out => control_flags_out,
		
		link_start => link_start,
		link_disable => link_disable,
		auto_start => auto_start,
		link_status => link_status,
		error_status => error_status,
		txclk_sel => txclk_sel,
		
		dout => dout,
		sout => sout,
		din => din,
		sin => sin
	);

	--spw if
	tx_fifo_we <= '0';--'0' when tx_fifo_data_num="111111" else '1';
	tx_fifo_data_in <= (others => '0');
	rx_fifo_oe <= '0';
	
	--
	-- spacewire control
	--
	process (clk100meg, reset) begin
		if (reset = '1') then
			init_wait_count <= 0;
			link_start <= '0';
			link_disable <= '0';
			auto_start <= '0';
		elsif (clk100meg'event and clk100meg = '1') then
			if (init_wait_count < 500) then
				init_wait_count <= init_wait_count + 1;
			else
				link_start <= '1';
				link_disable <= '0';
				auto_start <= '1';
			end if;
		end if;
	end process;

	txclk_sel <= "000000";


	--ebus
	ebus_adr <= (others => 'Z');
	ebus_d(15 downto 8) <= (others => 'Z');-- control_flag_in and time_in from UserFPGA
	control_flags_in <= ebus_d(15 downto 14);
	time_in <= ebus_d(13 downto 8);
	ebus_d(7 downto 0) <= control_flags_out & time_out;
	ebus_ena <= tick_out;
	
	ebus_wrn <= 'Z';
	ebus_rdn <= 'Z';
	tick_in <= ebus_done; -- tick_in from UserFPGA
	
	ebus_grant <= '0';
	ebus_done <= 'Z';

	--sdram
	cko <= 'Z';
	cke <= 'Z';
	ceb <= 'Z';
	rasb <= 'Z';
	casb <= 'Z';
	web <= 'Z';
	dm <= 'Z';
	addr <= (others => 'Z');
	ba <= (others => 'Z');
	dq <= (others => 'Z');
	
	led_process : process (clk)
	begin
		if(reset='1')then
			led_out(3 downto 0) <= (others=>'0');
		elsif(clk100meg'event and clk100meg='1')then
			if(counter<100000000)then
				led_out(2) <= '1';
				led_out(0) <= '1';
				led_out(1) <= rx_fifo_oe;
				led_out(3) <= tx_fifo_we;
				counter <= counter + 1;
			elsif(counter<200000000)then
				led_out(2) <= '0';
				led_out(0) <= '1';
				led_out(1) <= rx_fifo_data_num(1);
				led_out(3) <= rx_fifo_data_num(0);
				counter <= counter + 1;
			elsif(counter<300000000)then
				led_out(2) <= '1';
				led_out(0) <= '0';
				led_out(1) <= tx_fifo_data_num(1);
				led_out(3) <= tx_fifo_data_num(0);
				counter <= counter + 1;
			else
				counter <= 0;
			end if;
			
		end if;
	end process;
	
	test(3 downto 1) <= (others => '0');
	
	test(0) <= ebus_req;
end behavioral;

